import 'package:pull_to_refresh/pull_to_refresh.dart';
import './base_model.dart';

abstract class BaseListModel<T> extends BaseModel {
  // 当前页
  int _page = 1;

  // 页数据长度
  int _pageSize = 20;

  // 是否请求完毕
  bool _isRequestDone = false;

  // 是否加载失败
  bool _isLoadFail = false;

  // 是否加载完全部
  bool _isLoadAll = false;

  // 是否第一次加载
  bool _isFirstLoad = true;

  // 列表数据
  List<T> _list = [];

  // 列表数据2
  List<Map> _list2 = [];

  // 下拉刷新控制器
  RefreshController _refreshController = RefreshController();

  // 获取当前页
  int getPage() {
    return _page;
  }

  // 获取页数据长度
  int getPageSize() {
    return _pageSize;
  }

  //设置pageSize
  setPageSize(int size) {
    this._pageSize = size;
  }

  // 是否加载完毕
  bool isRequestDone() {
    return _isRequestDone;
  }

  // 是否记载失败
  bool isLoadFail() {
    return _isLoadFail;
  }

  // 是否首次加载
  bool isFirstLoad() {
    return _isFirstLoad;
  }

  // 是否加载完全部
  bool isLoadAll() {
    return _isLoadAll;
  }

  // 获取数据
  List<T> getList() {
    return _list;
  }

  List<Map> getList2() {
    return _list2;
  }

  // 获取下拉刷新控制器
  RefreshController getRefreshController() {
    return this._refreshController;
  }

  // 获取分页信息
  Map<String, dynamic> getPagination() {
    return {"page": getPage(), "pagesize": getPageSize()};
  }

  // 加载数据
  Future<void> getData() async {
    this._isRequestDone = false;

    try {
      List<T> list = await request();
      // 第一页清除旧数据
      if (getPage() == 1) {
        getList().replaceRange(0, getList().length, list);
      } else {
        getList().addAll(list);
      }
      if (list.length == 0 || list.length < getPageSize()) {
        List<Map> list2 = await request2();
        if (getPage() == 1) {
          getList2().replaceRange(0, getList2().length, list2);
        } else {
          getList2().addAll(list2);
        }
      }
      if (list.length > 0) {
        if (getPage() == 1) {
          getRefreshController().refreshCompleted();
          getRefreshController().loadComplete();
        } else {
          getRefreshController().loadComplete();
        }
        _page++;
        _isLoadAll = false;
      } else {
        _isLoadAll = true;
        if (getPage() == 1) {
          getRefreshController().refreshCompleted();
          getRefreshController().loadComplete();
        } else {
          getRefreshController().loadNoData();
        }
      }
    } catch (e) {
      _isLoadFail = true;
      if (getPage() == 1) {
        getRefreshController().refreshCompleted();
      } else {
        getRefreshController().loadFailed();
      }
    }
    _isRequestDone = true;
    _isFirstLoad = false;

    notifyListeners();
  }

  // 刷新数据
  Future<void> refresh() async {
    _page = 1;
    _isLoadAll = false;
    _isLoadFail = false;
    await getData();
    return null;
  }

  // 清除数据并通知更新，并滚动至顶部
  void clear() {
    getList().clear();
    _page = 1;
    _isLoadAll = false;
    _isLoadFail = false;
    _isRequestDone = false;
    notifyListeners();
  }

  // 请求数据2
  Future<List<Map>> request2() async {
    return [];
  }

  // 请求数据
  Future<List<T>> request();
}
