import 'package:flutter/material.dart';
import 'package:flutter/rendering.dart';

abstract class BaseModel extends ChangeNotifier {
  ///
  /// 当前model的状态[viewState]
  ///
  ViewState viewState = ViewState.idle;

  // 关联provider上下文
  BuildContext? _context;
  // widget是否摧毁
  bool _disposed = true;

  // 关联provider上下文
  void attachContext(BuildContext context) {
    _disposed = false;
    _context = context;
  }

  BuildContext? getContext() {
    return _context;
  }

  // 列表滚至顶部
  void scrollTop() {
    if (_context != null) {
      RenderSliver? renderSliver =
          _context!.findAncestorRenderObjectOfType<RenderSliver>();
      if (renderSliver != null &&
          (renderSliver.constraints.overlap > 0 ||
              renderSliver.constraints.cacheOrigin < 0)) {
        Scrollable.ensureVisible(_context!);
      }
    }
  }

  @override
  void dispose() {
    _disposed = true;
    super.dispose();
  }

  @override
  void notifyListeners() {
    // widget未被摧毁数据更新通知
    if (!_disposed) {
      super.notifyListeners();
    }
  }
}

/// 页面状态类型
enum ViewState {
  idle,
  busy, //加载中
  empty, //无数据
  error, //加载失败
  unAuthorized, //未登录
}
