import 'package:flutter/material.dart';
import 'package:provider/provider.dart';
import '../models/base_model.dart';
import '../actions/base_action.dart';

class QMProvider<T extends BaseModel> extends StatelessWidget {
  final T model;

  final Widget Function(
    BuildContext context,
    T model,
    Widget? child,
  ) builderWidget;

  QMProvider.value({
    Key? key,
    required this.model,
    required this.builderWidget,
  });

  @override
  Widget build(BuildContext context) {
    model.attachContext(context);
    return ChangeNotifierProvider<T>.value(
      key: key,
      value: model,
      child: Consumer<T>(
        builder: builderWidget,
      ),
    );
  }
}

class ActionProvider<T extends BaseModel, A extends BaseAction<T>>
    extends StatefulWidget {
  final A action;
  final Widget Function(BuildContext context, Widget? child) builder;
  ActionProvider({Key? key, required this.action, required this.builder});
  @override
  State<StatefulWidget> createState() => _ActionProviderState<T, A>();
}

class _ActionProviderState<T extends BaseModel?, A extends BaseAction<T>>
    extends State<ActionProvider> {
  @override
  void initState() {
    super.initState();
    widget.action.initState();
  }

  @override
  Widget build(BuildContext context) {
    return ChangeNotifierProvider<T?>.value(
      value: widget.action.getModel() as T?,
      child: Consumer<T>(
        builder: (context, vm, child) {
          widget.action.initBuilder(context);
          return widget.builder(context, child);
        },
      ),
    );
  }

  @override
  void dispose() {
    widget.action.dispose();
    super.dispose();
  }
}
